@@ -198,7 +198,7 @@ static int decompress_zstd(const char *inbuf, char *outbuf, u64 compress_len,
 
 	zret = ZSTD_decompressStream(strm, &out, &in);
 	if (ZSTD_isError(zret)) {
-		error("zstd decompress failed %s\n", ZSTD_getErrorName(zret));
+		error("zstd decompress failed %s", ZSTD_getErrorName(zret));
 		ret = -1;
 		goto out;
 	}
@@ -325,7 +325,7 @@ static int copy_one_inline(struct btrfs_root *root, int fd,
 	ram_size = btrfs_file_extent_ram_bytes(leaf, fi);
 	outbuf = calloc(1, ram_size);
 	if (!outbuf) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		return -ENOMEM;
 	}
 
@@ -391,18 +391,16 @@ static int copy_one_extent(struct btrfs_root *root, int fd,
 		size_left -= offset;
 	}
 
-	pr_verbose(offset ? 1 : 0, "offset is %llu\n", offset);
-
 	inbuf = malloc(size_left);
 	if (!inbuf) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		return -ENOMEM;
 	}
 
 	if (compress != BTRFS_COMPRESS_NONE) {
 		outbuf = calloc(1, ram_size);
 		if (!outbuf) {
-			error_msg(ERROR_MSG_MEMORY, NULL);
+			error_mem(NULL);
 			free(inbuf);
 			return -ENOMEM;
 		}
@@ -1028,7 +1026,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 			char *dir = strdup(fs_name);
 
 			if (!dir) {
-				error_msg(ERROR_MSG_MEMORY, NULL);
+				error_mem(NULL);
 				ret = -ENOMEM;
 				goto out;
 			}
